;; Generated package description from lentic-server.el  -*- no-byte-compile: t -*-
(define-package "lentic-server" "0.2.0.20240314.214448" "Web Server for Emacs Literate Source" '((lentic "0.8") (web-server "0.1.1")) :commit "732b88e7a183707ba65c38e8b3517cac42572644" :url "https://elpa.gnu.org/packages/lentic-server.html" :authors '(("Phillip Lord" . "phillip.lord@newcastle.ac.uk")) :maintainer '("Phillip Lord" . "phillip.lord@newcastle.ac.uk"))
